#include <stdio.h>
#include <string.h>

#include "codegen.h"

#include "symtab.h"

extern _codelist codelist;

extern long lnLongs, lnFunctions, lnPieces, lnStaticVar;

extern _symtab symtab[6];

extern long *plnLongsInFunction;

typedef struct _COBHeader
	{
	long	lVersion;
	long	lNumFunctions;
	long	lNumPieces;
	long	lLenCode; //# of longs in the script code
	long	lNumStaticVar;
	long	lAlways0;
	long	lOffsetFunctionCodeIndexArray;
	long	lOffsetFunctionNameIndexArray;
	long	lOffsetPieceNameIndexArray;
	long	lOffsetCodeStart;
	long	lOffsetFunctionNames;
	}_COBHeader; //0x2C

extern bool bDLLLinkData;
extern char *pszDLLLinkData;

extern long	lSizeDLLLinkData;

long assemblecob(char *psz)
	{
	FILE *fp = fopen(psz, "wb");

	_codelist *p = &codelist;

	long lVersion = 0x7; //ta:k is 0x6, so we'll make this 0x7
	long lAlways0 = 0x0;

	fwrite(&lVersion, 4, 1, fp);
	fwrite(&lnFunctions, 4, 1, fp);
	fwrite(&lnPieces, 4, 1, fp);
	fwrite(&lnLongs, 4, 1, fp);
	fwrite(&lnStaticVar, 4, 1, fp);
	fwrite(&lAlways0, 4, 1, fp);

	long lOffsetToFunctionCodeIndexArray;
	long lOffsetToFunctionNameOffsetArray;
	long lOffsetToPieceNameOffsetArray;
	long lOffsetToCodeStart = 0x2C;
	long lOffsetToFunctionNames;

	long lModified = lSizeDLLLinkData % 4; //make the dll link data size evenly divisible by 4
	if (lModified)
		lModified = 4 - (lSizeDLLLinkData % 4);

	lOffsetToFunctionCodeIndexArray = lOffsetToCodeStart + (lnLongs * 4) + lSizeDLLLinkData + lModified; //ok
	lOffsetToFunctionNameOffsetArray = lOffsetToFunctionCodeIndexArray + (lnFunctions * 4);
	lOffsetToPieceNameOffsetArray = lOffsetToFunctionNameOffsetArray + (lnFunctions * 4);
	lOffsetToFunctionNames = lOffsetToPieceNameOffsetArray + (lnPieces * 4);

	fwrite(&lOffsetToFunctionCodeIndexArray, 4, 1, fp);
	fwrite(&lOffsetToFunctionNameOffsetArray, 4, 1, fp);
	fwrite(&lOffsetToPieceNameOffsetArray, 4, 1, fp);
	fwrite(&lOffsetToCodeStart, 4, 1, fp);
	fwrite(&lOffsetToFunctionNames, 4, 1, fp);

	while (p)
		{
		fwrite(&p->lCode, 4, 1, fp);
		p = p->pNext;
		}

	//write DLL Link Data
	fwrite(pszDLLLinkData, 1, lSizeDLLLinkData, fp);
	fwrite("", 1, lModified, fp);

	//write function code index array
	long lOffset = 0;
	for (long i = 0; i < lnFunctions; i++)
		{
		fwrite(&lOffset, 4, 1, fp);
		lOffset += plnLongsInFunction[i];
		}


	//write function name offset array
	_symtab *ps = &symtab[2];
	lOffset = lOffsetToFunctionNames;
	for (long j = 0; j < lnFunctions; j++)
		{
		fwrite(&lOffset, 4, 1, fp);
		lOffset += strlen(ps->tok.psz) + 1;
		ps = ps->pNext;
		}

	//write piece name offset array
	ps = &symtab[0];
	for (long k = 0; k < lnPieces; k++)
		{
		fwrite(&lOffset, 4, 1, fp);
		lOffset += strlen(ps->tok.psz) + 1;
		ps = ps->pNext;
		}

	char ch = 0;

	//write function name array
	ps = &symtab[2];
	for (long l = 0; l < lnFunctions; l++)
		{
		fwrite(ps->tok.psz, 1, strlen(ps->tok.psz), fp);
		fwrite(&ch, 1, 1, fp);
		ps = ps->pNext;
		}

	//write piece name array
	ps = &symtab[0];
	for (long m = 0; m < lnPieces; m++)
		{
		fwrite(ps->tok.psz, 1, strlen(ps->tok.psz), fp);
		fwrite(&ch, 1, 1, fp);
		ps = ps->pNext;
		}

	fclose(fp);
	return 0;
	}